# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

import pytest
import openai
# from devtools_testutils import AzureRecordedTestCase
from conftest import configure, AZURE, OPENAI, ALL


class TestCompletions:
    """Missing tests for keyword argument `suffix`"""

    @pytest.mark.parametrize("api_type", [AZURE])
    @configure
    def test_completion_bad_deployment_name(self, azure_openai_creds, api_type):
        with pytest.raises(openai.error.InvalidRequestError) as e:
            openai.Completion.create(prompt="hello world", deployment_id="deployment")
        assert e.value.http_status == 404
        assert "The API deployment for this resource does not exist" in str(e.value)

    @pytest.mark.parametrize("api_type", [AZURE])
    @configure
    def test_completion_kw_input(self, azure_openai_creds, api_type):
        deployment = azure_openai_creds["completions_name"]

        completion = openai.Completion.create(prompt="hello world", deployment_id=deployment)
        assert completion
        completion = openai.Completion.create(prompt="hello world", engine=deployment)
        assert completion
        with pytest.raises(openai.error.InvalidRequestError) as e:
            openai.Completion.create(prompt="hello world", model=deployment)
        assert "Must provide an 'engine' or 'deployment_id' parameter" in str(e.value)

    @pytest.mark.parametrize("api_type", ALL)
    @configure
    def test_completion(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}
        kwargs = {'deployment_id': 'gpt-35-turbo-16k', 'max_tokens': 200, 'presence_penalty': 0.6, 'stream': False,
                  'temperature': 0.9, 'messages': [{"role": "user", "content": "1+1=?"}]}
        completion = openai.Completion.create(prompt="hello world", **kwargs)
        assert completion.id
        assert completion.object == "text_completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].text

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_batched_completions(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        completion = openai.Completion.create(prompt=["hello world", "how are you today?"], **kwargs)
        assert completion.id
        assert completion.object == "text_completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 2
        for c in completion.choices:
            assert c.finish_reason
            assert c.index is not None
            assert c.text

    @pytest.mark.skip(
        "openai.error.APIError: Invalid response object from API: 'Unsupported data type\n' (HTTP response code was 400)")
    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_completion_token_input(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        completion = openai.Completion.create(prompt=[10919, 3124, 318, 281, 17180, 30], **kwargs)
        assert completion.id
        assert completion.object == "text_completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 2
        for c in completion.choices:
            assert c.finish_reason
            assert c.index is not None
            assert c.text

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_streamed_completions(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        response = openai.Completion.create(prompt="hello world", stream=True, **kwargs)

        for completion in response:
            # API versions after 2023-05-15 send an empty first completion with RAI
            if len(completion.choices) > 0:
                assert completion.id
                assert completion.object == "text_completion"
                assert completion.created
                assert completion.model
                for c in completion.choices:
                    assert c.index is not None
                    assert c.text is not None

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_completion_max_tokens(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        completion = openai.Completion.create(
            prompt="How do I bake a chocolate cake?",
            max_tokens=50,
            **kwargs
        )

        assert completion.id
        assert completion.object == "text_completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens <= 50
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].text

    @pytest.mark.parametrize("api_type", [AZURE])
    @configure
    def test_completion_content_filter_prompt(self, azure_openai_creds, api_type):
        deployment = azure_openai_creds["completions_name"]

        with pytest.raises(openai.error.InvalidRequestError) as e:
            openai.Completion.create(
                prompt="How do I rob a bank?",
                deployment_id=deployment,
            )
        assert e.value.http_status == 400
        assert e.value.error.code == "content_filter"
        assert "The response was filtered due to the prompt triggering Azure OpenAI’s content management policy" in str(
            e.value)

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_completion_temperature(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        completion = openai.Completion.create(
            prompt="How do I bake a chocolate cake?",
            temperature=0.8,
            **kwargs
        )

        assert completion.id
        assert completion.object == "text_completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].text

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_completion_top_p(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        completion = openai.Completion.create(
            prompt="How do I bake a chocolate cake?",
            top_p=0.1,
            **kwargs
        )

        assert completion.id
        assert completion.object == "text_completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].text

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_completion_n(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        completion = openai.Completion.create(
            prompt="hello world",
            n=3,
            **kwargs
        )

        assert completion.id
        assert completion.object == "text_completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 3
        for idx, c in enumerate(completion.choices):
            assert c.index == idx
            assert c.text

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_completion_logprobs(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        completion = openai.Completion.create(
            prompt="How do I bake a chocolate cake?",
            logprobs=2,
            **kwargs
        )

        assert completion.id
        assert completion.object == "text_completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].text
        assert completion.choices[0].logprobs.tokens
        assert completion.choices[0].logprobs.token_logprobs
        assert completion.choices[0].logprobs.top_logprobs
        assert completion.choices[0].logprobs.text_offset

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_completion_echo(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        prompt = "How do I bake a chocolate cake?"
        completion = openai.Completion.create(
            prompt=prompt,
            echo=True,
            **kwargs
        )

        assert completion.id
        assert completion.object == "text_completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert prompt in completion.choices[0].text

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_completion_stop(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        completion = openai.Completion.create(
            prompt="How do I bake a chocolate cake?",
            stop=" ",
            **kwargs
        )

        assert completion.id
        assert completion.object == "text_completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].index is not None
        assert completion.choices[0].text

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_completion_token_penalty(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        completion = openai.Completion.create(
            prompt="How do I bake a chocolate cake?",
            presence_penalty=2,
            frequency_penalty=2,
            **kwargs
        )

        assert completion.id
        assert completion.object == "text_completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].text

    @pytest.mark.parametrize("api_type", [AZURE])
    @configure
    def test_completion_best_of(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        completion = openai.Completion.create(
            prompt="How do I bake a chocolate cake?",
            best_of=2,
            max_tokens=50,
            **kwargs
        )

        assert completion.id
        assert completion.object == "text_completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].text

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_completion_user(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        completion = openai.Completion.create(
            prompt="How do I bake a chocolate cake?",
            user="krista",
            **kwargs
        )

        assert completion.id
        assert completion.object == "text_completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].text

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_completion_logit_bias(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        completion = openai.Completion.create(
            prompt="What color is the ocean?",
            logit_bias={17585: -100, 14573: -100},
            **kwargs
        )

        assert completion.id
        assert completion.object == "text_completion"
        assert completion.created
        assert completion.model
        assert completion.usage.completion_tokens is not None
        assert completion.usage.prompt_tokens is not None
        assert completion.usage.total_tokens == completion.usage.completion_tokens + completion.usage.prompt_tokens
        assert len(completion.choices) == 1
        assert completion.choices[0].finish_reason
        assert completion.choices[0].index is not None
        assert completion.choices[0].text

    @pytest.mark.parametrize("api_type", [AZURE])
    @configure
    def test_completion_rai_annotations(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["completions_model"]} if api_type == "openai" \
            else {"deployment_id": azure_openai_creds["completions_name"]}

        # prompt filtered
        with pytest.raises(openai.error.InvalidRequestError) as e:
            completion = openai.Completion.create(
                prompt="how do I rob a bank?",
                **kwargs
            )
        assert e.value.code == "content_filter"
        content_filter_result = e.value.error.innererror.content_filter_result
        assert content_filter_result.hate.filtered is False
        assert content_filter_result.hate.severity == "safe"
        assert content_filter_result.self_harm.filtered is False
        assert content_filter_result.self_harm.severity == "safe"
        assert content_filter_result.sexual.filtered is False
        assert content_filter_result.sexual.severity == "safe"
        assert content_filter_result.violence.filtered is True
        assert content_filter_result.violence.severity is not None

        # not filtered
        completion = openai.Completion.create(
            prompt="What color is the ocean?",
            **kwargs
        )
        # prompt content filter result
        prompt_filter_result = completion.prompt_annotations[0].content_filter_results
        assert prompt_filter_result.hate.filtered is False
        assert prompt_filter_result.hate.severity == "safe"
        assert prompt_filter_result.self_harm.filtered is False
        assert prompt_filter_result.self_harm.severity == "safe"
        assert prompt_filter_result.sexual.filtered is False
        assert prompt_filter_result.sexual.severity == "safe"
        assert prompt_filter_result.violence.filtered is False
        assert prompt_filter_result.violence.severity == "safe"

        # output content filter result
        output_filter_result = completion.choices[0].content_filter_results
        assert output_filter_result.hate.filtered is False
        assert output_filter_result.hate.severity == "safe"
        assert output_filter_result.self_harm.filtered is False
        assert output_filter_result.self_harm.severity == "safe"
        assert output_filter_result.sexual.filtered is False
        assert output_filter_result.sexual.severity == "safe"
        assert output_filter_result.violence.filtered is False
        assert output_filter_result.violence.severity == "safe"



import dataclasses
from typing import Dict, List


@dataclasses.dataclass
class ModelParams:
    model: str = None
    prompt: List[Dict[str, str]] = None
    top_p: float = None
    temperature: float = None

    def asdict(self):
        return dataclasses.asdict(self)



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

import pytest
import openai
from devtools_testutils import AzureRecordedTestCase
from conftest import configure_async, AZURE, OPENAI, ALL


class TestEmbeddingsAsync(AzureRecordedTestCase):

    @pytest.mark.asyncio
    @pytest.mark.parametrize("api_type", [AZURE])
    @configure_async
    async def test_embedding_bad_deployment_name(self, azure_openai_creds, api_type):
        with pytest.raises(openai.error.InvalidRequestError) as e:
            await openai.Embedding.acreate(input="hello world", deployment_id="deployment")
        assert e.value.http_status == 404
        assert "The API deployment for this resource does not exist" in str(e.value)

    @pytest.mark.asyncio
    @pytest.mark.parametrize("api_type", [AZURE])
    @configure_async
    async def test_embedding_kw_input(self, azure_openai_creds, api_type):
        deployment = azure_openai_creds["embeddings_name"]

        embedding = await openai.Embedding.acreate(input="hello world", deployment_id=deployment)
        assert embedding
        embedding = await openai.Embedding.acreate(input="hello world", engine=deployment)
        assert embedding
        with pytest.raises(openai.error.InvalidRequestError) as e:
            await openai.Embedding.acreate(input="hello world", model=deployment)
        assert "Must provide an 'engine' or 'deployment_id' parameter" in str(e.value)

    @pytest.mark.asyncio
    @pytest.mark.parametrize("api_type", ALL)
    @configure_async
    async def test_embedding(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["embeddings_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["embeddings_name"]}

        embedding = await openai.Embedding.acreate(input="hello world", **kwargs)
        assert embedding.object == "list"
        assert embedding.model
        assert embedding.usage.prompt_tokens is not None
        assert embedding.usage.total_tokens is not None
        assert len(embedding.data) == 1
        assert embedding.data[0].object == "embedding"
        assert embedding.data[0].index is not None
        assert len(embedding.data[0].embedding) > 0

    @pytest.mark.skip("openai.error.InvalidRequestError: Too many inputs. The max number of inputs is 1. "
                      "We hope to increase the number of inputs per request soon. Please contact us through "
                      "an Azure support request at: https://go.microsoft.com/fwlink/?linkid=2213926 for further questions")
    @pytest.mark.asyncio
    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure_async
    async def test_embedding_batched(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["embeddings_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["embeddings_name"]}
        embedding = await openai.Embedding.acreate(input=["hello world", "second input"], **kwargs)

    @pytest.mark.asyncio
    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure_async
    async def test_embedding_user(self, azure_openai_creds, api_type):
        kwargs = {"model": azure_openai_creds["embeddings_model"]} if api_type == "openai" \
          else {"deployment_id": azure_openai_creds["embeddings_name"]}

        embedding = await openai.Embedding.acreate(input="hello world", user="krista", **kwargs)
        assert embedding.object == "list"
        assert embedding.model
        assert embedding.usage.prompt_tokens is not None
        assert embedding.usage.total_tokens is not None
        assert len(embedding.data) == 1
        assert embedding.data[0].object == "embedding"
        assert embedding.data[0].index is not None
        assert len(embedding.data[0].embedding) > 0



import json
import threading
import uuid
from typing import Dict, Any, List

import requests

import dwai
from dwai.bailian.model_api.models import ChatQaMessage
from dwai.bailian.model_api.exceptions import CompletionsRequestError

_thread_context = threading.local()

class BaseCompletions:
    """ 调用百联进行文本生成 """
    app_id = ""
    app_key = ""
    api_base = ""

    def __init__(self, app_id: str, app_key: str, api_base: str):
        self.app_id = app_id
        self.app_key = app_key
        self.api_base = api_base

    @classmethod
    def call(cls, *args, **kwargs) -> Any:
        raise NotImplementedError()

    def do_http_v1(self, prompt: str, history: List[ChatQaMessage] = None, stream: bool = False, **kwargs):

        headers = dict()
        headers["Content-Type"] = "application/json;charset=UTF-8"
        headers["Authorization"] = "Bearer %s" % self.app_key
        headers["User-Agent "] = "dw"

        if stream:
            headers["Accept"] = "text/event-stream"

        self.validate(prompt=prompt)

        h = []
        if history is not None:
            for v in history:
                h.append(v.to_dict())

        data = {
            "model": "qwen-plus-v1",
            "input": {
                "prompt": prompt,
                "history": h
            },
            "parameters": kwargs
        }

        url = "%s%s" % (self.api_base, "/bailian/services/aigc/text-generation/generation")
        session = self.__get_session()
        resp = session.request("POST",
                               url=url,
                               headers=headers,
                               data=json.dumps(data),
                               stream=stream)

        if not resp.ok:
            raise CompletionsRequestError("completion request error", resp.status_code, resp.text)

        return resp

    @staticmethod
    def __get_session() -> requests.Session:
        if not hasattr(_thread_context, "session"):
            _thread_context.session = requests.Session()

        return _thread_context.session

    def validate(self, prompt: str):

        if not self.api_base:
            raise ValueError("you need to get api base")

        if not self.app_key:
            raise ValueError("you need to set DW_API_KEY before calling api")

        if not self.app_id:
            raise ValueError("app id is required")

        if not prompt:
            raise ValueError("prompt is required")






import json
import threading
from typing import Any

import requests
from dwai.core.exceptions import CompletionsRequestError

_thread_context = threading.local()

class BaseCompletions:
    """ 对话API基类 """
    app_id = ""
    app_key = ""
    api_base = ""

    def __init__(self, app_id: str, app_key: str, api_base: str):
        self.app_id = app_id
        self.app_key = app_key
        self.api_base = api_base

    @classmethod
    def call(cls, *args, **kwargs) -> Any:
        raise NotImplementedError()

    def do_http(self, api_path, headers, data, stream):
        url = "%s%s" % (self.api_base, api_path)
        session = self.__get_session()
        resp = session.request("POST",
                               url=url,
                               headers=headers,
                               data=json.dumps(data),
                               stream=stream)

        if not resp.ok:
            raise CompletionsRequestError("completion request error", resp.status_code, resp.text)

        return resp

    @staticmethod
    def __get_session() -> requests.Session:
        if not hasattr(_thread_context, "session"):
            _thread_context.session = requests.Session()

        return _thread_context.session







# coding: utf-8

import six

class Message:

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    sensitive_list = []

    openapi_types = {
        'role': 'str',
        'content': 'str'
    }

    attribute_map = {
        'role': 'role',
        'content': 'content'
    }

    def __init__(self, role=None, content=None):
        """Message

        The model defined in huaweicloud sdk

        :param role: 角色
        :type role: str
        :param content: 问答对文本内容
        :type content: str
        """



        self._role = None
        self._content = None
        self.discriminator = None

        if role is not None:
            self.role = role
        self.content = content

    @property
    def role(self):
        """Gets the role of this Message.

        角色

        :return: The role of this Message.
        :rtype: str
        """
        return self._role

    @role.setter
    def role(self, role):
        """Sets the role of this Message.

        角色

        :param role: The role of this Message.
        :type role: str
        """
        self._role = role

    @property
    def content(self):
        """Gets the content of this Message.

        问答对文本内容

        :return: The content of this Message.
        :rtype: str
        """
        return self._content

    @content.setter
    def content(self, content):
        """Sets the content of this Message.

        问答对文本内容

        :param content: The content of this Message.
        :type content: str
        """
        self._content = content

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                if attr in self.sensitive_list:
                    result[attr] = "****"
                else:
                    result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        import simplejson as json
        if six.PY2:
            import sys
            reload(sys)
            sys.setdefaultencoding("utf-8")
        return json.dumps(self, ensure_ascii=False)

    def __repr__(self):
        """For `print`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Message):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
